package com.fairphone.hiccup;

import com.fairphone.hiccup.IHiccupdService;
import com.fairphone.hiccup.LogFile;

import android.os.RemoteException;
import android.os.ServiceManager;
import android.util.Log;
import java.util.List;



public class HiccupdService {
  private static final String TAG = "HiccupdService";
  private final IHiccupdService mBinder;

  private HiccupdService(IHiccupdService binder) {
    mBinder = binder;
  }

  public static HiccupdService getInstance() {
    IHiccupdService HiccupdService = IHiccupdService.Stub.asInterface(
      ServiceManager.getService("com.fairphone.hiccupd"));
    return new HiccupdService(HiccupdService);
  }

  public List<LogFile> getLogs() {
    try {
      return mBinder.getLogs();
    } catch (RemoteException e) {
      Log.w(TAG, "Cannot connect to Hiccupd Service", e);
      return null;
    }
  }

  public List<LogFile> getLastKmsg() {
    try {
      return mBinder.getLastKmsg();
    } catch (RemoteException e) {
      Log.w(TAG, "Cannot connect to Hiccupd Service", e);
      return null;
    }
  }

  public void cleanLogs() {
    try {
      mBinder.cleanLogs();
    } catch (RemoteException e) {
      Log.e(TAG, "Cannot connect to Hiccupd Service", e);
    }
  }

  public void startLogging() {
    try {
      mBinder.startLogging();
    } catch (RemoteException e) {
      Log.e(TAG, "Cannot connect to Hiccupd Service", e);
    }
  }

  public void stopLogging() {
    try {
       mBinder.stopLogging();
    } catch (RemoteException e) {
      Log.e(TAG, "Cannot connect to Hiccupd Service", e);
    }
  }

  public String getBootReason() {
    try {
       return mBinder.getBootReason();
    } catch (RemoteException e) {
      Log.e(TAG, "Cannot connect to Hiccupd Service", e);
      return null;
    }
  }
  public String getPowerOnReason() {
    try {
       return mBinder.getPowerOnReason();
    } catch (RemoteException e) {
      Log.e(TAG, "Cannot connect to Hiccupd Service", e);
      return null;
    }
  }
  public String getPowerOffReason() {
    try {
       return mBinder.getPowerOffReason();
    } catch (RemoteException e) {
      Log.e(TAG, "Cannot connect to Hiccupd Service", e);
      return null;
    }
  }

  public int getApiVersion() {
    try {
       return mBinder.getApiVersion();
    } catch (RemoteException e) {
      Log.e(TAG, "Cannot connect to Hiccupd Service", e);
      return 0;
    }
  }
}
