#include "BpHiccupd.hpp"

using namespace android;

namespace hiccup {
#define PLOG aout
#define HICCUPD_INTERFACE_NAME "com.fairphone.IHiccupd"

IMPLEMENT_META_INTERFACE(Hiccupd, HICCUPD_INTERFACE_NAME);

sp<IHiccupd> getHiccupService() {
  sp<IServiceManager> s = defaultServiceManager();
  sp<IBinder> binder = s->getService(String16("com.fairphone.hiccupd"));
  if (binder == 0)
    return 0;
  sp<IHiccupd> hiccupd = interface_cast<IHiccupd>(binder);
  return hiccupd;
}

Vector<LogFile> *BpHiccupd::getLastKmsg() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  Vector<LogFile> *v = new Vector<LogFile>();
  int ret = remote()->transact(GET_LAST_KMESG, data, &reply);
  int num_logs = reply.readInt32();
  for (int i = 0; i < num_logs; i++) {
    String16 path = reply.readString16();
    int fd = dup(reply.readFileDescriptor());
    v->push_back(LogFile(path,fd));
  }
  return v;
}

Vector<LogFile> *BpHiccupd::getLogs() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  Vector<LogFile> *v = new Vector<LogFile>();
  int ret = remote()->transact(GET_LOGS, data, &reply);
  int num_logs = reply.readInt32();
  for (int i = 0; i < num_logs; i++) {
    String16 path = reply.readString16();
    int fd = dup(reply.readFileDescriptor());
    v->push_back(LogFile(path,fd));
  }
  return v;
}

status_t BpHiccupd::cleanLogs() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  int ret = remote()->transact(CLEAN_LOGS, data, &reply);
  return ret;
}

status_t BpHiccupd::startLogging() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  int ret = remote()->transact(START_LOGGING, data, &reply);
  return ret;
}

status_t BpHiccupd::stopLogging() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  int ret = remote()->transact(STOP_LOGGING, data, &reply);
  return ret;
}

String16 BpHiccupd::getBootReason() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  int ret = remote()->transact(GET_BOOTREASON, data, &reply);
  return reply.readString16();
}

String16 BpHiccupd::getPowerOnReason() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  int ret = remote()->transact(GET_POWERONREASON, data, &reply);
  return reply.readString16();
}

String16 BpHiccupd::getPowerOffReason() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  int ret = remote()->transact(GET_POWEROFFREASON, data, &reply);
  return reply.readString16();
}

int32_t BpHiccupd::getApiVersion() {
  Parcel data, reply;
  data.writeInterfaceToken(String16(HICCUPD_INTERFACE_NAME));
  int ret = remote()->transact(GET_API_VERSION, data, &reply);
  return reply.readInt32();
}

} /* namespace hiccup */
