#include "BnHiccupd.hpp"
#include <stdio.h>
#include <stdlib.h>

#include <sys/stat.h>
#include <fcntl.h>
#include <binder/IPCThreadState.h>
#include  "glob.h"

using namespace android;
using namespace hiccup;

class Hiccupd:public BnHiccupd {
  Vector<LogFile> *getLogs();
  Vector<LogFile> *getLastKmsg();
  status_t cleanLogs();
  status_t startLogging();
  status_t stopLogging();
  String16 getBootReason();
  String16 getPowerOnReason();
  String16 getPowerOffReason();
  int32_t  getApiVersion();
  int readIntFromFile(const char *fileName);
};

Vector<LogFile> *Hiccupd::getLastKmsg() {
  Vector<LogFile> *v = new Vector<LogFile>();

  /* make local copy of logs */
  system("rm -rf /cache/hiccup");
  system("mkdir -p /cache/hiccup");
  system("cp /proc/last_kmsg /cache/hiccup/");

  glob_t globres;
  if ( glob("/cache/hiccup/*", 0, NULL, &globres) == !OK) {
    return NULL;
  }

  for(unsigned int i=0;i<globres.gl_pathc;++i){
    int fd;
    if ( (fd= open(globres.gl_pathv[i], O_RDONLY)) >= 0 ) {
      v->push_back(LogFile(globres.gl_pathv[i], fd));
    }
  }

out:
  globfree(&globres);
  system("rm -rf /cache/hiccup/");
  return v;
}

Vector<LogFile> *Hiccupd::getLogs() {
  Vector<LogFile> *v = new Vector<LogFile>();

  /* make local copy of logs */
  system("rm -rf /cache/hiccup");
  system("mkdir -p /cache/hiccup");
  system("cp /data/hiccup/logcat* /cache/hiccup/");
  system("cp /proc/last_kmsg /cache/hiccup/_last_kmsg");

  glob_t globres;
  if ( glob("/cache/hiccup/*", 0, NULL, &globres) == !OK) {
    return NULL;
  }

  for(unsigned int i=0;i<globres.gl_pathc;++i){
    int fd;
    if ( (fd= open(globres.gl_pathv[i], O_RDONLY)) >= 0 ) {
      v->push_back(LogFile(globres.gl_pathv[i], fd));
    }
  }

out:
  globfree(&globres);
  system("rm -rf /cache/hiccup/");
  return v;
}

status_t Hiccupd::cleanLogs() {
  system("rm -rf /data/hiccup/*");
  return OK;
}

status_t Hiccupd::startLogging() {
  system("setprop persist.fp.hiccup.logging 1");
  return OK;
}

status_t Hiccupd::stopLogging() {
  system("setprop persist.fp.hiccup.logging 0");
  return OK;
}

#define BUF_SIZE 16
int Hiccupd::readIntFromFile(const char *fileName) {
  int fd = open(fileName, O_RDONLY);
  char buf[BUF_SIZE+1];
  ssize_t bytes_read = 0;
  int ret = -1;
  if ( !( (fd < 0) || ( (bytes_read = read(fd,buf, BUF_SIZE) ) == 0)) ) {
    buf[bytes_read] = 0;
    ret = atoi(buf);
  }
  if (fd <0) {
    close(fd);
  }
  return ret;
}

String16 Hiccupd::getBootReason() {
  String16 ret;
  int boot_reason = readIntFromFile("/proc/sys/kernel/boot_reason");
  switch (boot_reason) {
    case 0x01: ret = String16("keyboard power on"); break;
    case 0x02: ret = String16("RTC alarm"); break;
    case 0x04: ret = String16("cable power on"); break;
    case 0x08: ret = String16("SMPL"); break;
    case 0x10: ret = String16("Watch Dog timeout"); break;
    case 0x20: ret = String16("USB charger"); break;
    case 0x40: ret = String16("Wall charger"); break;
    case 0xFF: ret = String16("error reading power_on_status value"); break;
    default:   ret = String16(String8::format("Unkown: %d", boot_reason));

  }
  return ret;
}

String16 Hiccupd::getPowerOnReason() {
  String16 ret;
  int power_reason = readIntFromFile("/proc/sys/qpnp-power-on/pon_reason");
  switch (power_reason) {
    case 0:  ret = String16("(0) Triggered from Hard Reset"); break;
    case 1:  ret = String16("(1) Triggered from SMPL (sudden momentary power loss)"); break;
    case 2:  ret = String16("(2) Triggered from RTC (RTC alarm expiry)"); break;
    case 3:  ret = String16("(3) Triggered from DC (DC charger insertion)"); break;
    case 4:  ret = String16("(4) Triggered from USB (USB charger insertion)"); break;
    case 5:  ret = String16("(5) Triggered from PON1 (secondary PMIC)"); break;
    case 6:  ret = String16("(6) Triggered from CBL (external power supply)"); break;
    case 7:  ret = String16("(7) Triggered from KPD (power key press)"); break;
    default: ret = String16(String8::format("(%d) Unkown", power_reason));
  }
  return ret;
}

String16 Hiccupd::getPowerOffReason() {
  String16 ret;
  int power_reason = readIntFromFile("/proc/sys/qpnp-power-on/poff_reason");
  switch (power_reason) {
    case 0:  ret = String16("(0) Triggered from SOFT (Software)"); break;
    case 1:  ret = String16("(1) Triggered from PS_HOLD (PS_HOLD/MSM controlled shutdown)"); break;
    case 2:  ret = String16("(2) Triggered from PMIC_WD (PMIC watchdog)"); break;
    case 3:  ret = String16("(3) Triggered from GP1 (Keypad_Reset1)"); break;
    case 4:  ret = String16("(4) Triggered from GP2 (Keypad_Reset2)"); break;
    case 5:  ret = String16("(5) Triggered from KPDPWR_AND_RESIN (Simultaneous power key and reset line)"); break;
    case 6:  ret = String16("(6) Triggered from RESIN_N (Reset line/Volume Down Key)"); break;
    case 7:  ret = String16("(7) Triggered from KPDPWR_N (Long Power Key hold)"); break;
    case 8:  ret = String16("(8) N/A 8"); break;
    case 9:  ret = String16("(9) N/A 9"); break;
    case 10: ret = String16("(10) N/A 10"); break;
    case 11: ret = String16("(11) Triggered from CHARGER (Charger ENUM_TIMER, BOOT_DONE)"); break;
    case 12: ret = String16("(12) Triggered from TFT (Thermal Fault Tolerance)"); break;
    case 13: ret = String16("(13) Triggered from UVLO (Under Voltage Lock Out)"); break;
    case 14: ret = String16("(14) Triggered from OTST3 (Overtemp)"); break;
    case 15: ret = String16("(15) Triggered from STAGE3 (Stage 3 reset)"); break;
    default: ret = String16(String8::format("(%d) Unkown", power_reason));
  }
  return ret;
}

int32_t Hiccupd::getApiVersion() {
  return HICCUP_API_VERSION;
}

int main() {
  defaultServiceManager()->addService(String16("com.fairphone.hiccupd"), new Hiccupd());
  ProcessState::self()->startThreadPool();
  IPCThreadState::self()->joinThreadPool();
  return 0;
}
