#pragma once
/*
 * Author: Dirk Vogt <dirk@fairphone.com>
 *
 * Hiccup deamon
 *
 * This deamon is the priviledged part of hiccup and exports the logcat files
 * and last_kmsg to the unpriviledged java service.
 */

#include <stdlib.h>

#include <utils/RefBase.h>
#include <utils/Log.h>
#include <binder/TextOutput.h>

#include <binder/IInterface.h>
#include <binder/IBinder.h>
#include <binder/Parcel.h>
#include <binder/ProcessState.h>
#include <binder/IServiceManager.h>

#define LOGTAG "Hiccupd"

using namespace android;

#define HICCUP_API_VERSION 2

namespace hiccup {

class LogFile {
public:
  String16 fileName;
  int fd;
  LogFile(): fileName(""),fd(-1) {}
  LogFile(String16 path, int fd): fileName(path), fd(fd) {}
  LogFile(char *path, int fd): fileName(path), fd(fd) {}
  ~LogFile() {}
};

class IHiccupd : public IInterface {
    public:
        enum {
            GET_LOGS   = IBinder::FIRST_CALL_TRANSACTION,
            CLEAN_LOGS,
            START_LOGGING,
            STOP_LOGGING,
            GET_BOOTREASON,
            GET_POWERONREASON,
            GET_POWEROFFREASON,
            GET_LAST_KMESG,
            GET_API_VERSION,
        };

        virtual Vector<LogFile> *getLogs()     = 0;
        virtual Vector<LogFile> *getLastKmsg() = 0;
        virtual status_t cleanLogs()           = 0;
        virtual status_t startLogging()        = 0;
        virtual status_t stopLogging()         = 0;
        virtual String16 getBootReason()       = 0;
        virtual String16 getPowerOnReason()    = 0;
        virtual String16 getPowerOffReason()   = 0;
        virtual int32_t  getApiVersion()       = 0;

        // prepares the logs for the caller
        DECLARE_META_INTERFACE(Hiccupd);
    private:
};

} /* namespace hiccup */
